@@ -1,6 +1,7 @@
 import abc
 import logging
-from typing import Callable, Generic, List, TypeVar
+import threading
+from typing import Awaitable, Callable, Generic, List, Optional, TypeVar
 
 import av
 import numpy as np
@@ -9,7 +10,76 @@ from aiortc.contrib.media import MediaPlayer, MediaRecorder
 logger = logging.getLogger(__name__)
 
 
-class VideoProcessorBase(abc.ABC):
+VideoFrameCallback = Callable[[av.VideoFrame], av.VideoFrame]
+QueuedVideoFramesCallback = Callable[
+    [List[av.VideoFrame]], Awaitable[List[av.VideoFrame]]
+]
+AudioFrameCallback = Callable[[av.AudioFrame], av.AudioFrame]
+QueuedAudioFramesCallback = Callable[
+    [List[av.AudioFrame]], Awaitable[List[av.AudioFrame]]
+]
+MediaEndedCallback = Callable[[], None]
+
+
+class ProcessorBase(abc.ABC):
+    def recv(self, frame: av.VideoFrame) -> av.VideoFrame:
+        raise NotImplementedError()
+
+    async def recv_queued(self, frames: List[av.VideoFrame]) -> List[av.VideoFrame]:
+        raise NotImplementedError()
+
+    def on_ended(self):
+        raise NotImplementedError()
+
+
+class CallbackAttachableProcessor(ProcessorBase):
+    _frame_callback: Optional[VideoFrameCallback]
+    _queued_frames_callback: Optional[QueuedVideoFramesCallback]
+    _media_ended_callback: Optional[MediaEndedCallback]
+
+    def __init__(
+        self,
+        frame_callback: Optional[VideoFrameCallback],
+        queued_frames_callback: Optional[QueuedVideoFramesCallback],
+        ended_callback: Optional[MediaEndedCallback],
+    ) -> None:
+        self._lock = threading.Lock()
+        self._frame_callback = frame_callback
+        self._queued_frames_callback = queued_frames_callback
+        self._media_ended_callback = ended_callback
+
+    def update_callbacks(
+        self,
+        frame_callback: Optional[VideoFrameCallback],
+        queued_frames_callback: Optional[QueuedVideoFramesCallback],
+        ended_callback: Optional[MediaEndedCallback],
+    ) -> None:
+        with self._lock:
+            self._frame_callback = frame_callback
+            self._queued_frames_callback = queued_frames_callback
+            self._media_ended_callback = ended_callback
+
+    def recv(self, frame: av.VideoFrame) -> av.VideoFrame:
+        with self._lock:
+            if self._frame_callback:
+                return self._frame_callback(frame)
+
+        return frame
+
+    async def recv_queued(self, frames: List[av.VideoFrame]) -> List[av.VideoFrame]:
+        with self._lock:
+            if self._queued_frames_callback:
+                return await self._queued_frames_callback(frames)
+
+        return [self.recv(frames[-1])]
+
+    def on_ended(self):
+        with self._lock:
+            if self._media_ended_callback:
+                return self._media_ended_callback()
+
+
+class VideoProcessorBase(ProcessorBase):
     """
     A base class for video processors.
     """
@@ -36,7 +106,7 @@ class VideoProcessorBase(abc.ABC):
         new_image = self.transform(frame)
         return av.VideoFrame.from_ndarray(new_image, format="bgr24")
 
-    async def recv_queued(self, frames: List[av.VideoFrame]) -> av.VideoFrame:
+    async def recv_queued(self, frames: List[av.VideoFrame]) -> List[av.VideoFrame]:
         """
         Receives all the frames arrived after the previous recv_queued() call
         and returns new frames when running in async mode.
@@ -59,7 +129,7 @@ class VideoTransformerBase(VideoProcessorBase):  # Backward compatiblity
     """
 
 
-class AudioProcessorBase(abc.ABC):
+class AudioProcessorBase(ProcessorBase):
     """
     A base class for audio processors.
     """
@@ -75,7 +145,7 @@ class AudioProcessorBase(abc.ABC):
         """
         raise NotImplementedError("recv() is not implemented.")
 
-    async def recv_queued(self, frames: List[av.AudioFrame]) -> av.AudioFrame:
+    async def recv_queued(self, frames: List[av.AudioFrame]) -> List[av.AudioFrame]:
         """
         Receives all the frames arrived after the previous recv_queued() call
         and returns new frames when running in async mode.
@@ -102,7 +172,7 @@ MediaRecorderFactory = Callable[[], MediaRecorder]
 VideoProcessorFactory = Callable[[], VideoProcessorT]
 AudioProcessorFactory = Callable[[], AudioProcessorT]
 
-ProcessorT = TypeVar("ProcessorT", VideoProcessorBase, AudioProcessorBase)
+ProcessorT = TypeVar("ProcessorT", bound=ProcessorBase)
 FrameT = TypeVar("FrameT", av.VideoFrame, av.AudioFrame)
 
 
